@@ -29,7 +29,6 @@ from typing import (
     TYPE_CHECKING,
     Literal,
     Mapping,
-    Optional,
     Sequence,
     TypeVar,
     overload,
@@ -80,26 +79,26 @@ def parse_integer(
 
 
 @overload
-def parse_integer(request: Request, name: str, *, negative: bool) -> Optional[int]: ...
+def parse_integer(request: Request, name: str, *, negative: bool) -> int | None: ...
 
 
 @overload
 def parse_integer(
     request: Request,
     name: str,
-    default: Optional[int] = None,
+    default: int | None = None,
     required: bool = False,
     negative: bool = False,
-) -> Optional[int]: ...
+) -> int | None: ...
 
 
 def parse_integer(
     request: Request,
     name: str,
-    default: Optional[int] = None,
+    default: int | None = None,
     required: bool = False,
     negative: bool = False,
-) -> Optional[int]:
+) -> int | None:
     """Parse an integer parameter from the request string
 
     Args:
@@ -136,8 +135,8 @@ def parse_integer_from_args(
 def parse_integer_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[int] = None,
-) -> Optional[int]: ...
+    default: int | None = None,
+) -> int | None: ...
 
 
 @overload
@@ -153,19 +152,19 @@ def parse_integer_from_args(
 def parse_integer_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[int] = None,
+    default: int | None = None,
     required: bool = False,
     negative: bool = False,
-) -> Optional[int]: ...
+) -> int | None: ...
 
 
 def parse_integer_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[int] = None,
+    default: int | None = None,
     required: bool = False,
     negative: bool = False,
-) -> Optional[int]:
+) -> int | None:
     """Parse an integer parameter from the request string
 
     Args:
@@ -217,13 +216,13 @@ def parse_boolean(request: Request, name: str, *, required: Literal[True]) -> bo
 
 @overload
 def parse_boolean(
-    request: Request, name: str, default: Optional[bool] = None, required: bool = False
-) -> Optional[bool]: ...
+    request: Request, name: str, default: bool | None = None, required: bool = False
+) -> bool | None: ...
 
 
 def parse_boolean(
-    request: Request, name: str, default: Optional[bool] = None, required: bool = False
-) -> Optional[bool]:
+    request: Request, name: str, default: bool | None = None, required: bool = False
+) -> bool | None:
     """Parse a boolean parameter from the request query string
 
     Args:
@@ -265,17 +264,17 @@ def parse_boolean_from_args(
 def parse_boolean_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[bool] = None,
+    default: bool | None = None,
     required: bool = False,
-) -> Optional[bool]: ...
+) -> bool | None: ...
 
 
 def parse_boolean_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[bool] = None,
+    default: bool | None = None,
     required: bool = False,
-) -> Optional[bool]:
+) -> bool | None:
     """Parse a boolean parameter from the request query string
 
     Args:
@@ -318,8 +317,8 @@ def parse_boolean_from_args(
 def parse_bytes_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[bytes] = None,
-) -> Optional[bytes]: ...
+    default: bytes | None = None,
+) -> bytes | None: ...
 
 
 @overload
@@ -336,17 +335,17 @@ def parse_bytes_from_args(
 def parse_bytes_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[bytes] = None,
+    default: bytes | None = None,
     required: bool = False,
-) -> Optional[bytes]: ...
+) -> bytes | None: ...
 
 
 def parse_bytes_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[bytes] = None,
+    default: bytes | None = None,
     required: bool = False,
-) -> Optional[bytes]:
+) -> bytes | None:
     """
     Parse a string parameter as bytes from the request query string.
 
@@ -380,7 +379,7 @@ def parse_string(
     name: str,
     default: str,
     *,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
 ) -> str: ...
 
@@ -391,7 +390,7 @@ def parse_string(
     name: str,
     *,
     required: Literal[True],
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
 ) -> str: ...
 
@@ -401,21 +400,21 @@ def parse_string(
     request: Request,
     name: str,
     *,
-    default: Optional[str] = None,
+    default: str | None = None,
     required: bool = False,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[str]: ...
+) -> str | None: ...
 
 
 def parse_string(
     request: Request,
     name: str,
-    default: Optional[str] = None,
+    default: str | None = None,
     required: bool = False,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[str]:
+) -> str | None:
     """
     Parse a string parameter from the request query string.
 
@@ -455,10 +454,10 @@ def parse_string(
 def parse_json(
     request: Request,
     name: str,
-    default: Optional[dict] = None,
+    default: dict | None = None,
     required: bool = False,
     encoding: str = "ascii",
-) -> Optional[JsonDict]:
+) -> JsonDict | None:
     """
     Parse a JSON parameter from the request query string.
 
@@ -492,10 +491,10 @@ def parse_json(
 def parse_json_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[dict] = None,
+    default: dict | None = None,
     required: bool = False,
     encoding: str = "ascii",
-) -> Optional[JsonDict]:
+) -> JsonDict | None:
     """
     Parse a JSON parameter from the request query string.
 
@@ -559,9 +558,9 @@ def parse_enum(
     request: Request,
     name: str,
     E: type[EnumT],
-    default: Optional[EnumT] = None,
+    default: EnumT | None = None,
     required: bool = False,
-) -> Optional[EnumT]:
+) -> EnumT | None:
     """
     Parse an enum parameter from the request query string.
 
@@ -601,7 +600,7 @@ def parse_enum(
 
 def _parse_string_value(
     value: bytes,
-    allowed_values: Optional[StrCollection],
+    allowed_values: StrCollection | None,
     name: str,
     encoding: str,
 ) -> str:
@@ -627,9 +626,9 @@ def parse_strings_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
     *,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[list[str]]: ...
+) -> list[str] | None: ...
 
 
 @overload
@@ -638,7 +637,7 @@ def parse_strings_from_args(
     name: str,
     default: list[str],
     *,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
 ) -> list[str]: ...
 
@@ -649,7 +648,7 @@ def parse_strings_from_args(
     name: str,
     *,
     required: Literal[True],
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
 ) -> list[str]: ...
 
@@ -658,22 +657,22 @@ def parse_strings_from_args(
 def parse_strings_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[list[str]] = None,
+    default: list[str] | None = None,
     *,
     required: bool = False,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[list[str]]: ...
+) -> list[str] | None: ...
 
 
 def parse_strings_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[list[str]] = None,
+    default: list[str] | None = None,
     required: bool = False,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[list[str]]:
+) -> list[str] | None:
     """
     Parse a string parameter from the request query string list.
 
@@ -720,21 +719,21 @@ def parse_strings_from_args(
 def parse_string_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[str] = None,
+    default: str | None = None,
     *,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[str]: ...
+) -> str | None: ...
 
 
 @overload
 def parse_string_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[str] = None,
+    default: str | None = None,
     *,
     required: Literal[True],
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
 ) -> str: ...
 
@@ -743,21 +742,21 @@ def parse_string_from_args(
 def parse_string_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[str] = None,
+    default: str | None = None,
     required: bool = False,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[str]: ...
+) -> str | None: ...
 
 
 def parse_string_from_args(
     args: Mapping[bytes, Sequence[bytes]],
     name: str,
-    default: Optional[str] = None,
+    default: str | None = None,
     required: bool = False,
-    allowed_values: Optional[StrCollection] = None,
+    allowed_values: StrCollection | None = None,
     encoding: str = "ascii",
-) -> Optional[str]:
+) -> str | None:
     """
     Parse the string parameter from the request query string list
     and return the first result.
@@ -812,12 +811,12 @@ def parse_json_value_from_request(
 @overload
 def parse_json_value_from_request(
     request: Request, allow_empty_body: bool = False
-) -> Optional[JsonDict]: ...
+) -> JsonDict | None: ...
 
 
 def parse_json_value_from_request(
     request: Request, allow_empty_body: bool = False
-) -> Optional[JsonDict]:
+) -> JsonDict | None:
     """Parse a JSON value from the body of a twisted HTTP request.
 
     Args:
@@ -980,8 +979,8 @@ class ResolveRoomIdMixin:
         self.room_member_handler = hs.get_room_member_handler()
 
     async def resolve_room_id(
-        self, room_identifier: str, remote_room_hosts: Optional[list[str]] = None
-    ) -> tuple[str, Optional[list[str]]]:
+        self, room_identifier: str, remote_room_hosts: list[str] | None = None
+    ) -> tuple[str, list[str] | None]:
         """
         Resolve a room identifier to a room ID, if necessary.
 
