@@ -18,7 +18,6 @@
 # [This file includes modifications made by New Vector Limited]
 #
 #
-from typing import Dict
 
 from synapse.api.constants import EventTypes, Membership
 from synapse.events import EventBase
@@ -56,8 +55,8 @@ async def get_badge_count(store: DataStore, user_id: str, group_by_room: bool) -
 
 async def get_context_for_event(
     storage: StorageControllers, ev: EventBase, user_id: str
-) -> Dict[str, str]:
-    ctx: Dict[str, str] = {}
+) -> dict[str, str]:
+    ctx: dict[str, str] = {}
 
     if ev.internal_metadata.outlier:
         # We don't have state for outliers, so we can't compute the context
