@@ -21,7 +21,7 @@
 
 import logging
 from inspect import isawaitable
-from typing import TYPE_CHECKING, Any, Generic, Optional, TypeVar, cast
+from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast
 
 import attr
 from txredisapi import (
@@ -53,6 +53,7 @@ from synapse.replication.tcp.protocol import (
     tcp_inbound_commands_counter,
     tcp_outbound_commands_counter,
 )
+from synapse.util.duration import Duration
 
 if TYPE_CHECKING:
     from synapse.replication.tcp.handler import ReplicationCommandHandler
@@ -72,10 +73,10 @@ class ConstantProperty(Generic[T, V]):
 
     constant: V = attr.ib()
 
-    def __get__(self, obj: Optional[T], objtype: Optional[type[T]] = None) -> V:
+    def __get__(self, obj: T | None, objtype: type[T] | None = None) -> V:
         return self.constant
 
-    def __set__(self, obj: Optional[T], value: V) -> None:
+    def __set__(self, obj: T | None, value: V) -> None:
         pass
 
 
@@ -119,7 +120,7 @@ class RedisSubscriber(SubscriberProtocol):
 
         # a logcontext which we use for processing incoming commands. We declare it as a
         # background process so that the CPU stats get reported to prometheus.
-        self._logging_context: Optional[BackgroundProcessLoggingContext] = None
+        self._logging_context: BackgroundProcessLoggingContext | None = None
 
     def _get_logging_context(self) -> BackgroundProcessLoggingContext:
         """
@@ -293,14 +294,14 @@ class SynapseRedisFactory(RedisFactory):
         self,
         hs: "HomeServer",
         uuid: str,
-        dbid: Optional[int],
+        dbid: int | None,
         poolsize: int,
         isLazy: bool = False,
         handler: type = ConnectionHandler,
         charset: str = "utf-8",
-        password: Optional[str] = None,
+        password: str | None = None,
         replyTimeout: int = 30,
-        convertNumbers: Optional[int] = True,
+        convertNumbers: int | None = True,
     ):
         super().__init__(
             uuid=uuid,
@@ -317,7 +318,7 @@ class SynapseRedisFactory(RedisFactory):
         self.hs = hs  # nb must be called this for @wrap_as_background_process
         self.server_name = hs.hostname
 
-        hs.get_clock().looping_call(self._send_ping, 30 * 1000)
+        hs.get_clock().looping_call(self._send_ping, Duration(seconds=30))
 
     @wrap_as_background_process("redis_ping")
     async def _send_ping(self) -> None:
@@ -422,9 +423,9 @@ def lazyConnection(
     hs: "HomeServer",
     host: str = "localhost",
     port: int = 6379,
-    dbid: Optional[int] = None,
+    dbid: int | None = None,
     reconnect: bool = True,
-    password: Optional[str] = None,
+    password: str | None = None,
     replyTimeout: int = 30,
 ) -> ConnectionHandler:
     """Creates a connection to Redis that is lazily set up and reconnects if the
@@ -471,9 +472,9 @@ def lazyConnection(
 def lazyUnixConnection(
     hs: "HomeServer",
     path: str = "/tmp/redis.sock",
-    dbid: Optional[int] = None,
+    dbid: int | None = None,
     reconnect: bool = True,
-    password: Optional[str] = None,
+    password: str | None = None,
     replyTimeout: int = 30,
 ) -> ConnectionHandler:
     """Creates a connection to Redis that is lazily set up and reconnects if the
