@@ -26,7 +26,6 @@ from typing import (
     Any,
     Awaitable,
     Callable,
-    Optional,
     TypeVar,
 )
 
@@ -285,9 +284,9 @@ class BackfillStream(Stream):
         event_id: str
         room_id: str
         type: str
-        state_key: Optional[str]
-        redacts: Optional[str]
-        relates_to: Optional[str]
+        state_key: str | None
+        redacts: str | None
+        relates_to: str | None
 
     NAME = "backfill"
     ROW_TYPE = BackfillStreamRow
@@ -435,7 +434,7 @@ class ReceiptsStream(_StreamFromIdGen):
         receipt_type: str
         user_id: str
         event_id: str
-        thread_id: Optional[str]
+        thread_id: str | None
         data: dict
 
     NAME = "receipts"
@@ -510,7 +509,7 @@ class CachesStream(Stream):
         """
 
         cache_func: str
-        keys: Optional[list[Any]]
+        keys: list[Any] | None
         invalidation_ts: int
 
     NAME = "caches"
@@ -639,7 +638,7 @@ class AccountDataStream(_StreamFromIdGen):
     @attr.s(slots=True, frozen=True, auto_attribs=True)
     class AccountDataStreamRow:
         user_id: str
-        room_id: Optional[str]
+        room_id: str | None
         data_type: str
 
     NAME = "account_data"
