@@ -27,8 +27,6 @@ from typing import (
     Any,
     Counter,
     Iterable,
-    Optional,
-    Union,
     cast,
 )
 
@@ -296,7 +294,7 @@ class StatsStore(StateDeltasStore):
     @cached()
     async def get_earliest_token_for_stats(
         self, stats_type: str, id: str
-    ) -> Optional[int]:
+    ) -> int | None:
         """
         Fetch the "earliest token". This is used by the room stats delta
         processor to ignore deltas that have been processed between the
@@ -362,7 +360,7 @@ class StatsStore(StateDeltasStore):
         stats_id: str,
         fields: dict[str, int],
         complete_with_stream_id: int,
-        absolute_field_overrides: Optional[dict[str, int]] = None,
+        absolute_field_overrides: dict[str, int] | None = None,
     ) -> None:
         """
         Updates the statistics for a subject, with a delta (difference/relative
@@ -400,7 +398,7 @@ class StatsStore(StateDeltasStore):
         stats_id: str,
         fields: dict[str, int],
         complete_with_stream_id: int,
-        absolute_field_overrides: Optional[dict[str, int]] = None,
+        absolute_field_overrides: dict[str, int] | None = None,
     ) -> None:
         if absolute_field_overrides is None:
             absolute_field_overrides = {}
@@ -585,7 +583,7 @@ class StatsStore(StateDeltasStore):
             )
             return
 
-        room_state: dict[str, Union[None, bool, str]] = {
+        room_state: dict[str, None | bool | str] = {
             "join_rules": None,
             "history_visibility": None,
             "encryption": None,
@@ -680,12 +678,12 @@ class StatsStore(StateDeltasStore):
         self,
         start: int,
         limit: int,
-        from_ts: Optional[int] = None,
-        until_ts: Optional[int] = None,
-        order_by: Optional[str] = UserSortOrder.USER_ID.value,
+        from_ts: int | None = None,
+        until_ts: int | None = None,
+        order_by: str | None = UserSortOrder.USER_ID.value,
         direction: Direction = Direction.FORWARDS,
-        search_term: Optional[str] = None,
-    ) -> tuple[list[tuple[str, Optional[str], int, int]], int]:
+        search_term: str | None = None,
+    ) -> tuple[list[tuple[str, str | None, int, int]], int]:
         """Function to retrieve a paginated list of users and their uploaded local media
         (size and number). This will return a json list of users and the
         total number of users matching the filter criteria.
@@ -710,7 +708,7 @@ class StatsStore(StateDeltasStore):
 
         def get_users_media_usage_paginate_txn(
             txn: LoggingTransaction,
-        ) -> tuple[list[tuple[str, Optional[str], int, int]], int]:
+        ) -> tuple[list[tuple[str, str | None, int, int]], int]:
             filters = []
             args: list = []
 
@@ -782,7 +780,7 @@ class StatsStore(StateDeltasStore):
 
             args += [limit, start]
             txn.execute(sql, args)
-            users = cast(list[tuple[str, Optional[str], int, int]], txn.fetchall())
+            users = cast(list[tuple[str, str | None, int, int]], txn.fetchall())
 
             return users, count
 
